function EmpCopMSDI

% This code computes the the Standardized Precipitation Index (SPI), 
% Standardized Soil Moisture Index (SSI), and Multivariate Standardized Drought Index (MSDI)
% empirically (MSDIe) and parametrically (MSDIp). 
%
% "td.txt" includes sample monthly precipitation and soil moisture data (>=30 years,vector).
%
% Reference Publications:
% MSDIp:  Hao Z., AghaKouchak A., 2013, Multivariate Standardized Drought Index: A Parametric Multi-Index Model, Advances in Water Resources, 57, 12-18, doi: 10.1016/j.advwatres.2013.03.009.
% MSDIe   Hao Z., AghaKouchak A., 2014, A Nonparametric Multivariate Multi-Index Drought Monitoring Framework, Journal of Hydrometeorology, 15, 89-101, doi:10.1175/JHM-D-12-0160.1. 
%         Hao Z., AghaKouchak A., Nakhjiri N., Farahmand A., 2014, Global Integrated Drought Monitoring and Prediction System, Scientific Data, 1:140001, 1-10, doi: 10.1038/sdata.2014.1.

% Developed by: Hydroclimate Research Group, University of Califnornia, Irvine
% Web:  http://drought.eng.uci.edu/msdi.html
% Last update:  8/24/2013


%% Compute SPI, SSI, MSDIe (nonparametric MSDI) and MSDIp (parametric MSDI) with monthly Precipitation and and Soil moisture data (ds.txt).

clc;clear all;close all;

%Load Precipitation and Soil Moisture Data
d=load('td.txt');

% Monthly precipitation vector (mm/day, mm/month, in/day, in/month)
dp=d(:,1);

%Monthly soil moisture vector 
ds=d(:,2);

%(1) Uncomment the below part to test the code using synthetic data
%   mu = [4 8]; Sigma = [.9 .4; .4 .3];
%   d = mvnrnd(mu, Sigma, 396);
%   figure(2);plot(d(:,1),d(:,2),'.');
%   dp=d(:,1);ds=d(:,2);

nm=length(dp);

% Specify time scale (e.g., 6-month SPI or MSDI)

sc=6;

% Initialize matrices of drought indices

SPI=zeros(nm,1);SSI=zeros(nm,1);MSDIe=zeros(nm,1);MSDIp=zeros(nm,1);

% Compute empirical SPI and SSI

SPI(1:sc-1,1)=nan;
SPI(sc:end,1)=SPIComp(dp,sc);

SSI(1:sc-1,1)=nan;
SSI(sc:end,1)=SPIComp(ds,sc);

% Compute empirical MSDI

MSDIe(1:sc-1,1)=nan;
MSDIe(sc:end,1)=Dat2EmpMSDI(dp,ds,sc);

% Compute parametric (copula-based) MSDI (change copula family if needed)

MSDIp(1:sc-1,1)=nan;
MSDIp(sc:end,1)=Dat2CopMSDI(dp,ds,sc,'Frank');

% Plot the four indices

figure(1)

subplot(4,1,1);plot(1:nm,SPI,'g','LineWidth',2);
legend('SPI','Orientation','horizontal')
subplot(4,1,2);plot(1:nm,SSI,'b','LineWidth',2)
legend('SSI','Orientation','horizontal')
subplot(4,1,3);plot(1:nm,MSDIe,'r-.','LineWidth',2)
legend('MSDIe','Orientation','horizontal')
subplot(4,1,4);plot(1:nm,MSDIp,'k-.','LineWidth',2)
legend('MSDIp','Orientation','horizontal')

xk=[1980:2:2012];n=length(xk);           % Start and end dates for the lable 

for i=1:n xt(i)=(xk(i)-1980).*12+1; end  % Set ticks
for i=1:n xl{i}=num2str(xk(i)); end      % Label for each year

set(gca,'xtick',xt,'xticklabel',xl);
xlabel('Year');ylabel('Index');xlim([1 397])

figure(2)
% Plot all the four indices in one figure

plot(1:nm,SPI,'g','LineWidth',2);
hold on;plot(1:nm,SSI,'b','LineWidth',2)
hold on;plot(1:nm,MSDIe,'r-.','LineWidth',2)
hold on;plot(1:nm,MSDIp,'k-.','LineWidth',2)

xk=[1980:2:2012];n=length(xk);           % Start and end dates for the lable 

for i=1:n xt(i)=(xk(i)-1980).*12+1; end  % Set ticks
for i=1:n xl{i}=num2str(xk(i)); end      % Label for each year

set(gca,'xtick',xt,'xticklabel',xl);
xlabel('Year');ylabel('Index');xlim([1 397])
legend('SPI','SSI','MSDIe','MSDIp','Orientation','horizontal')
hold on;plot(1:nm,-0.8.*ones(nm,1),'r') % Threshold -0.8

% MSDIp is based on  Hao Z., AghaKouchak A., 2013, Multivariate Standardized Drought Index: A Parametric Multi-Index Model, Advances in Water Resources, 57, 12-18, doi: 10.1016/j.advwatres.2013.03.009.
% MSDIe is based on  Hao Z., AghaKouchak A., 2014, A Nonparametric Multivariate Multi-Index Drought Monitoring Framework, Journal of Hydrometeorology, 15, 89-101, doi:10.1175/JHM-D-12-0160.1. 

function SI=SPIComp(md,sc)

% (1) Computes accumulated precipitation and soil moisture data for the specific time scale
% (2) Computes empirical (and parametric - commented part) drought indicators (SPI and SSI)

% Compute accumulated precipitation and soil moisture for the time scale sc

A1=[];

for i=1:sc
    A1=[A1,md(i:length(md)-sc+i)];
end

Y=sum(A1,2);

% Compute SPI or SSI

n=length(Y);SI=zeros(n,1);

for k=1:12
    d=Y(k:12:n);
    
    % Parametric method
    
    %  [par,pc] = gamfit(d(d~=0));
    %  q=length(find(d==0))/length(d);
    %  SI(k:12:n,1)=q+(1-q).*gamcdf(d,par(1),par(2));
    
    %  Empirical method
    
    SI(k:12:n,1)=empdis(d);
end

SI(:,1)=norminv(SI(:,1));


function y=empdis(d)

%  Compute empirical probability

n=length(d);bp=zeros(n,1);

for i=1:n
    bp(i,1)=sum(d(:,1)<=d(i,1));
end

% Gringorten plotting position

y=(bp-0.44)./(n+0.12);

%  Can be replaced with the Weibull plotting position
% y=(bp)./(n+1);


function y=Dat2EmpMSDI(xd,yd,sc)

% (1) Compute accumulated prepitation and soil moisture for the specific time scale (sc)
% (2) Compute Empirical MSDI from the data

% Compute accumulated prepitation (xd) and soil moisture (yd) for the time scale sc

A1=[];B1=[];

for i=1:sc,
    A1=[A1,xd(i:length(xd)-sc+i)];
    B1=[B1,yd(i:length(yd)-sc+i)];
end

X=sum(A1,2);Y=sum(B1,2);

% Compute empirical MSDI

n=length(X);
p=zeros(n,1);

for k=1:12
    d1=X(k:12:n);d2=Y(k:12:n);
    p(k:12:n,1)=emp_biv([d1,d2]);
end

y=norminv(p);


function y=Dat2CopMSDI(xd,yd,sc,st)

% (1) Compute accumulated precipitation and soil moisture for the specific time scale (sc)
% (2) Compute MSDI from the data

% Compute accumulated precipitation (xd) and soil moisture (yd) for the scale sc

A1=[];B1=[];
for i=1:sc,
    A1=[A1,xd(i:length(xd)-sc+i)];
    B1=[B1,yd(i:length(yd)-sc+i)];
end

X=sum(A1,2);Y=sum(B1,2);

% Compute  MSDI

n=length(X);
SI=zeros(n,3);
cp=zeros(n,3);

for k=1:12
    
    copname=st;
    
    d1=X(k:12:n);
    d2=Y(k:12:n);
    
    %     Compute  marginal (empirical) probability (use Weibull)
    
    cp(k:12:n,1)=empdis(d1);
    cp(k:12:n,2)=empdis(d2);
    
    %     Fit copula
    
    theta=copulafit(copname,[cp(k:12:n,1),cp(k:12:n,2)]);
    
    %     Compute CDF
    
    cp(k:12:n,3)=copulacdf(copname,[cp(k:12:n,1),cp(k:12:n,2)],theta);
    
end

%  Standardized indicators (SPI,SSI and MSDI)

SI(:,1)=norminv(cp(:,1));
SI(:,2)=norminv(cp(:,2));
SI(:,3)=norminv(cp(:,3));

y=SI(:,3);



function y=emp_biv(d)

% Compute empirical probability

n=length(d);
bp=zeros(n,1);

for i=1:n
    
    td=zeros(n,3);
    
    td(d(:,1)<=d(i,1),1)=1;
    td(d(:,2)<=d(i,2),2)=1;
    
    td(:,3)=td(:,1).*td(:,2);
    bp(i,1)=sum(td(:,3));
    
end

% Gringorten plotting position (bivariate)

y=(bp-0.44)./(n+0.12);

% Can be replaced with Weibull plotting position (bivariate, or empirical copula)

% y=(bp)./(n+1);
